@@ -1,8 +1,21 @@
-# sql.R
+# file RODBC/R/sql.R
+# copyright (C) 1999-2007  M. Lapsley and B. D. Ripley
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 or 3 of the License
+#  (at your option).
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+#
 # high level functions for sql database access
 #
-#
-#
 ###########################################
 
 sqlDrop <- function(channel, sqtable, errors = TRUE)
@@ -506,7 +519,8 @@ sqlGetResults <-
     function (channel, as.is = FALSE,
               errors = FALSE, max = 0, buffsize = 1000,
               nullstring = NA, na.strings = "NA", believeNRows = TRUE,
-              dec = getOption("dec"))
+              dec = getOption("dec"),
+              stringsAsFactors = default.stringsAsFactors())
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
@@ -560,7 +574,9 @@ sqlGetResults <-
             else if(cData$type[i] == "timestamp")
                 data[[i]] <- as.POSIXct(data[[i]])
             else
-                data[[i]] <- type.convert(as.character(data[[i]]), dec = dec)
+                data[[i]] <- type.convert(as.character(data[[i]]),
+                                          as.is = !stringsAsFactors,
+                                          dec = dec)
         }
     }
     data
